<?php

class Shopware_Plugins_Core_SwagSecurity_Bootstrap extends Shopware_Components_Plugin_Bootstrap
{
    public function getCapabilities()
    {
        return array(
            'install' => true,
            'enable' => true,
            'update' => true,
            'secureUninstall' => false
        );
    }

    public function getLabel()
    {
        return 'Shopware Security Plugin';
    }

    public function getInfo()
    {
        return array(
            'version' => $this->getVersion(),
            'label' => $this->getLabel()
        );
    }

    public function getVersion()
    {
        $info = json_decode(file_get_contents(__DIR__ . '/plugin.json'), true);
        if ($info) {
            return $info['currentVersion'];
        } else {
            throw new Exception('The plugin has an invalid version file.');
        }
    }

    public function install()
    {
        $this->subscribeEvent('Enlight_Controller_Action_PreDispatch_Backend_ImportExport', 'fixExportOutput');
        $this->subscribeEvent('Enlight_Controller_Action_PreDispatch_Backend_Mail', 'fixMailSecurity');
        $this->subscribeEvent('Enlight_Controller_Action_PreDispatch_Backend_Widgets', 'fixWidgetsSecurity');
        $this->subscribeEvent('Shopware_Controllers_Backend_Order::getOrderDocumentsQuery::before', 'getOrderDocumentsQuery');
        $this->subscribeEvent('Shopware\Models\Article\Repository::getConfiguratorSetsWithExcludedIdsQueryBuilder::before', 'fixIdsArray');
        $this->subscribeEvent('Shopware\Models\Article\Repository::getArticlesWithExcludedIdsQueryBuilder::before', 'fixIdsArray');
        $this->subscribeEvent('Shopware\Models\Article\Repository::getSuppliersWithExcludedIdsQueryBuilder::before', 'fixIdsArray');
        $this->subscribeEvent('Shopware\Models\Blog\Repository::getListQueryBuilder::before', 'fixBlogRepositoryListQuery');
        $this->subscribeEvent('Shopware\Models\Blog\Repository::getFilterQueryBuilder::before', 'getFilterQueryBuilderBlogRepository');
        $this->subscribeEvent('Shopware\Models\Blog\Repository::getBackendListQueryBuilder::before', 'getBackendListQueryBuilderBlogRepository');
        $this->subscribeEvent('Shopware\Models\Customer\Repository::getCustomerDetailQueryBuilder::before', 'fixCustomerIdParameter');
        $this->subscribeEvent('Shopware\Models\Customer\Repository::getOrdersQueryBuilder::before', 'fixCustomerIdParameter');
        $this->subscribeEvent('Shopware\Models\Dispatch\Repository::getShippingCostsMatrixQueryBuilder::before', 'fixDispatchIdParameter');
        $this->subscribeEvent('Shopware\Models\Dispatch\Repository::getPaymentQueryBuilder::before', 'fixUsedIdsFilter');
        $this->subscribeEvent('Shopware\Models\Dispatch\Repository::getCountryQueryBuilder::before', 'fixUsedIdsFilter');
        $this->subscribeEvent('Shopware\Models\Dispatch\Repository::getHolidayQueryBuilder::before', 'fixUsedIdsFilter');
        $this->subscribeEvent('Shopware\Models\Media\Repository::getAlbumMediaQueryBuilder::before', 'fixMediaIdParameter');
        $this->subscribeEvent('Shopware\Models\Order\Document\Repository::getListQuery::before', 'fixOrderIdParameter');
        $this->subscribeEvent('Shopware\Models\Voucher\Repository::getValidateOrderCodeQueryBuilder::before', 'fixVoucherIdParameter');

        $form = $this->Form();
        $form->setElement('checkbox', 'fix_export_output', array(
            'label'    => 'Import/Export #18912',
            'description' => 'Escaping of the format request parameter',
            'value'    => 1
        ));

        $form->setElement('checkbox', 'fix_widgets_smarty_security', array(
            'label'    => 'Smarty Security Mail #18917',
            'description' => 'Enable template engine security for mail verify smarty call',
            'value'    => 1
        ));

        $form->setElement('checkbox', 'fix_mail_smarty_security', array(
            'label'    => 'Smarty Security Widgets #18917',
            'description' => 'Enable template engine security for send mail to merchant call',
            'value'    => 1
        ));

        $form->setElement('checkbox', 'fix_attribute_reset', array(
            'label'    => 'Attribute system security #18997',
            'description' => 'Escaping of table and column name parameter',
            'value'    => 1
        ));

        $form->setElement('checkbox', 'fix_sql_injection', array(
            'label'    => 'Possible sql injections #18915',
            'description' => 'Escaping different parameters in repositories',
            'value'    => 1
        ));

        return true;
    }

    public function fixVoucherIdParameter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $args->set('voucherId', (int) $args->get('voucherId'));
    }

    public function fixOrderIdParameter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $args->set('orderId', (int) $args->get('orderId'));
    }

    public function fixMediaIdParameter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $args->set('albumId', (int) $args->get('albumId'));
    }

    public function fixUsedIdsFilter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $filters = $args->get('filter');
        if (!is_array($filters)) {
            return;
        }

        $new = array();
        foreach ($filters as $key => $filter) {
            if ($filter['property'] == 'usedIds' || $filter['property'] == 'onlyIds') {
                $filter['value'] = $this->castIntArray($filter['value']);
            }
            $new[$key] = $filter;
        }
        $args->set('filter', $new);
    }

    public function fixDispatchIdParameter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $args->set('dispatchId', (int) $args->get('dispatchId'));
    }

    public function fixCustomerIdParameter(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $args->set('customerId', (int) $args->get('customerId'));
    }

    public function getBackendListQueryBuilderBlogRepository(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $ids = $args->get('blogCategoryIds');
        $ids = $this->castIntArray($ids);
        $args->set('blogCategoryIds', $ids);
    }

    public function getFilterQueryBuilderBlogRepository(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $ids = $args->get('categoryIds');
        $ids = $this->castIntArray($ids);
        $args->set('categoryIds', $ids);
    }

    public function fixBlogRepositoryListQuery(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $ids = $args->get('blogCategoryIds');
        $ids = $this->castIntArray($ids);
        $args->set('blogCategoryIds', $ids);
    }

    public function fixIdsArray(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $ids = $args->get('ids');
        $ids = $this->castIntArray($ids);
        $args->set('ids', $ids);
    }

    public function getOrderDocumentsQuery(Enlight_Hook_HookArgs $args)
    {
        if (!$this->Config()->get('fix_sql_injection')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        $ids = $args->get('orderIds');
        $ids = $this->castIntArray($ids);
        $args->set('orderIds', $ids);
    }

    public function fixWidgetsSecurity(Enlight_Event_EventArgs $args)
    {
        /** @var \Shopware_Controllers_Backend_Widgets $subject */
        $subject = $args->getSubject();
        $request = $subject->Request();

        if (strtolower($request->getActionName()) !== 'sendmailtomerchant') {
            return;
        }
        if (!$this->Config()->get('fix_widgets_smarty_security')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.26', '>=') ) {
            return;
        }
        include __DIR__ . '/Swag_Smarty_Security.php';

        $subject->View()->Engine()->enableSecurity(
            new Swag_Smarty_Security(
                $subject->View()->Engine(),
                $this->getSmartySecurityConfig()
            )
        );
    }

    public function fixMailSecurity(Enlight_Event_EventArgs $args)
    {
        /** @var \Shopware_Controllers_Backend_Mail $subject */
        $subject = $args->getSubject();
        $request = $subject->Request();

        if (strtolower($request->getActionName()) !== 'verifysmarty') {
            return;
        }
        if (!$this->Config()->get('fix_mail_smarty_security')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.26', '>=') ) {
            return;
        }
        include __DIR__ . '/Swag_Smarty_Security.php';

        $subject->View()->Engine()->enableSecurity(
            new Swag_Smarty_Security(
                $subject->View()->Engine(),
                $this->getSmartySecurityConfig()
            )
        );
    }

    public function fixExportOutput(Enlight_Event_EventArgs $args)
    {
        if (!$this->Config()->get('fix_export_output')) {
            return;
        }
        if (version_compare(Shopware::VERSION, '5.2.25', '>=') ) {
            return;
        }
        /** @var Enlight_Controller_Action $controller */
        $controller = $args->getSubject();

        $request = $controller->Request();

        $format = $request->getParam('format', null);
        if ($format !== null) {
            $request->setParam('format', htmlspecialchars($format));
        }

        $type = $request->getParam('type', null);
        if ($type !== null) {
            $request->setParam('type', htmlspecialchars($type));
        }
    }

    /**
     * @param $ids
     * @return array
     */
    private function castIntArray($ids)
    {
        if (!is_array($ids)) {
            $ids = array($ids);
        }
        $ids = array_map(function ($id) {
            return (int) $id;
        }, $ids);

        return $ids;
    }

    private function getSmartySecurityConfig()
    {
        $user = array();
        try {
            $user = Shopware()->getOption('template_security');
        } catch (\Exception $e) { }


        if (empty($user)) {
            try {
                $user = Shopware()->Container()->getParameter('shopware.template_security');
            } catch (\Exception $e) { }
        }

        $default = array(
            'php_modifiers' => include __DIR__ . '/smarty_functions.php',
            'php_functions' => include __DIR__ . '/smarty_functions.php',
        );

        if (empty($user)) {
            return $default;
        }

        return array_merge_recursive($default, $user);
    }
}